@@ -374,7 +374,8 @@ class DoNodeServiceTestCase(db_base.DbTestCase):
             target_provision_state=tgt_prov_state,
             last_error=None,
             driver_internal_info={'service_steps': self.service_steps,
-                                  'service_step_index': 0},
+                                  'service_step_index': 0,
+                                  'servicing_polling': True},
             service_step=self.service_steps[0])
         mock_execute.return_value = return_state
 
@@ -390,6 +391,7 @@ class DoNodeServiceTestCase(db_base.DbTestCase):
         self.assertEqual(1, node.driver_internal_info['service_step_index'])
         mock_execute.assert_called_once_with(
             mock.ANY, mock.ANY, self.service_steps[1])
+        self.assertNotIn('servicing_polling', node.driver_internal_info)
 
     def test_do_next_clean_step_continue_from_last_cleaning(self):
         self._do_next_clean_step_continue_from_last_cleaning(
@@ -1080,7 +1082,7 @@ class DoNodeCleanTestChildNodes(db_base.DbTestCase):
         child_node1.refresh()
         child_node2.refresh()
 
-        # Confirm the objects *did* recieve locks.
+        # Confirm the objects *did* receive locks.
         self.assertNotEqual(child1_updated_at, child_node1.updated_at)
         self.assertNotEqual(child2_updated_at, child_node2.updated_at)
 
@@ -1163,7 +1165,7 @@ class DoNodeCleanTestChildNodes(db_base.DbTestCase):
         child_node1.refresh()
         child_node2.refresh()
 
-        # Confirm the objects *did* recieve locks.
+        # Confirm the objects *did* receive locks.
         self.assertNotEqual(child1_updated_at, child_node1.updated_at)
         self.assertIsNone(child_node2.updated_at)
 
