@@ -26,10 +26,13 @@ from ironic.objects import bios
 from ironic.objects import chassis
 from ironic.objects import conductor
 from ironic.objects import deploy_template
+from ironic.objects import firmware
 from ironic.objects import node
 from ironic.objects import node_history
+from ironic.objects import node_inventory
 from ironic.objects import port
 from ironic.objects import portgroup
+from ironic.objects import runbook
 from ironic.objects import trait
 from ironic.objects import volume_connector
 from ironic.objects import volume_target
@@ -87,12 +90,6 @@ def get_test_ilo_info():
 
 def get_test_drac_info():
     return {
-        "drac_address": "1.2.3.4",
-        "drac_port": 443,
-        "drac_path": "/wsman",
-        "drac_protocol": "https",
-        "drac_username": "admin",
-        "drac_password": "fake",
         "redfish_address": "1.2.3.4",
         "redfish_system_id": "/redfish/v1/Systems/System.Embedded.1",
         "redfish_username": "admin",
@@ -164,7 +161,6 @@ def get_test_snmp_info(**kw):
 def get_test_node(**kw):
     properties = {
         "cpu_arch": "x86_64",
-        "cpus": "8",
         "local_gb": "10",
         "memory_mb": "4096",
     }
@@ -236,6 +232,9 @@ def get_test_node(**kw):
         'network_data': kw.get('network_data'),
         'boot_mode': kw.get('boot_mode', None),
         'secure_boot': kw.get('secure_boot', None),
+        'shard': kw.get('shard', None),
+        'parent_node': kw.get('parent_node', None),
+        'service_step': kw.get('service_step'),
     }
 
     for iface in drivers_base.ALL_INTERFACES:
@@ -258,7 +257,7 @@ def create_test_node(**kw):
     # Let DB generate an ID if one isn't specified explicitly.
     # Creating a node with tags or traits will raise an exception. If tags or
     # traits are not specified explicitly just delete them.
-    for field in {'id', 'tags', 'traits'}:
+    for field in {'id', 'uuid', 'tags', 'traits'}:
         if field not in kw:
             del node[field]
     dbapi = db_api.get_instance()
@@ -271,6 +270,8 @@ def get_test_port(**kw):
         'version': kw.get('version', port.Port.VERSION),
         'uuid': kw.get('uuid', '1be26c0b-03f2-4d2e-ae87-c02d7f33c781'),
         'node_id': kw.get('node_id', 123),
+        'node_uuid': kw.get('node_uuid',
+                            '59d102f7-5840-4299-8ec8-80c0ebae9de1'),
         'address': kw.get('address', '52:54:00:cf:2d:31'),
         'extra': kw.get('extra', {}),
         'created_at': kw.get('created_at'),
@@ -299,8 +300,9 @@ def create_test_port(**kw):
     """
     port = get_test_port(**kw)
     # Let DB generate ID if it isn't specified explicitly
-    if 'id' not in kw:
-        del port['id']
+    for field in ('id', 'uuid'):
+        if field not in kw:
+            del port[field]
     dbapi = db_api.get_instance()
     return dbapi.create_port(port)
 
@@ -331,8 +333,9 @@ def create_test_volume_connector(**kw):
     """
     connector = get_test_volume_connector(**kw)
     # Let DB generate ID if it isn't specified explicitly
-    if 'id' not in kw:
-        del connector['id']
+    for field in ('id', 'uuid'):
+        if field not in kw:
+            del connector[field]
     dbapi = db_api.get_instance()
     return dbapi.create_volume_connector(connector)
 
@@ -365,8 +368,9 @@ def create_test_volume_target(**kw):
     """
     target = get_test_volume_target(**kw)
     # Let DB generate ID if it isn't specified explicitly
-    if 'id' not in kw:
-        del target['id']
+    for field in ('id', 'uuid'):
+        if field not in kw:
+            del target[field]
     dbapi = db_api.get_instance()
     return dbapi.create_volume_target(target)
 
@@ -394,8 +398,9 @@ def create_test_chassis(**kw):
     """
     chassis = get_test_chassis(**kw)
     # Let DB generate ID if it isn't specified explicitly
-    if 'id' not in kw:
-        del chassis['id']
+    for field in ('id', 'uuid'):
+        if field not in kw:
+            del chassis[field]
     dbapi = db_api.get_instance()
     return dbapi.create_chassis(chassis)
 
@@ -409,6 +414,7 @@ def get_test_conductor(**kw):
         'conductor_group': kw.get('conductor_group', ''),
         'created_at': kw.get('created_at', timeutils.utcnow()),
         'updated_at': kw.get('updated_at', timeutils.utcnow()),
+        'online': kw.get('online', True)
     }
 
 
@@ -445,6 +451,8 @@ def get_test_portgroup(**kw):
         'uuid': kw.get('uuid', '6eb02b44-18a3-4659-8c0b-8d2802581ae4'),
         'name': kw.get('name', 'fooname'),
         'node_id': kw.get('node_id', 123),
+        'node_uuid': kw.get('node_uuid',
+                            '40481b96-306b-4a33-901f-795a3dc2f397'),
         'address': kw.get('address', '52:54:00:cf:2d:31'),
         'extra': kw.get('extra', {}),
         'created_at': kw.get('created_at'),
@@ -468,8 +476,9 @@ def create_test_portgroup(**kw):
     """
     portgroup = get_test_portgroup(**kw)
     # Let DB generate ID if it isn't specified explicitly
-    if 'id' not in kw:
-        del portgroup['id']
+    for field in ('id', 'uuid'):
+        if field not in kw:
+            del portgroup[field]
     dbapi = db_api.get_instance()
     return dbapi.create_portgroup(portgroup)
 
@@ -500,25 +509,6 @@ def create_test_node_tag(**kw):
     return dbapi.add_node_tag(tag['node_id'], tag['tag'])
 
 
-def get_test_xclarity_properties():
-    return {
-        "cpu_arch": "x86_64",
-        "cpus": "8",
-        "local_gb": "10",
-        "memory_mb": "4096",
-    }
-
-
-def get_test_xclarity_driver_info():
-    return {
-        'xclarity_manager_ip': "1.2.3.4",
-        'xclarity_username': "USERID",
-        'xclarity_password': "fake",
-        'xclarity_port': 443,
-        'xclarity_hardware_id': 'fake_sh_id',
-    }
-
-
 def get_test_node_trait(**kw):
     return {
         'version': kw.get('version', trait.Trait.VERSION),
@@ -684,15 +674,59 @@ def create_test_deploy_template(**kw):
     return dbapi.create_deploy_template(template)
 
 
-def get_test_ibmc_info():
+def get_test_runbook(**kw):
+    default_uuid = uuidutils.generate_uuid()
     return {
-        "ibmc_address": "https://example.com",
-        "ibmc_username": "username",
-        "ibmc_password": "password",
-        "verify_ca": False,
+        'version': kw.get('version', runbook.Runbook.VERSION),
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at'),
+        'id': kw.get('id', 234),
+        'name': kw.get('name', u'CUSTOM_DT1'),
+        'uuid': kw.get('uuid', default_uuid),
+        'steps': kw.get('steps', [get_test_runbook_step(
+            runbook_id=kw.get('id', 234))]),
+        'disable_ramdisk': kw.get('disable_ramdisk', False),
+        'extra': kw.get('extra', {}),
+        'public': kw.get('public', False),
+        'owner': kw.get('owner', None),
     }
 
 
+def get_test_runbook_step(**kw):
+    return {
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at'),
+        'id': kw.get('id', 345),
+        'runbook_id': kw.get('runbook_id', 234),
+        'interface': kw.get('interface', 'raid'),
+        'step': kw.get('step', 'create_configuration'),
+        'args': kw.get('args', {'logical_disks': []}),
+        'order': kw.get('order', 1)
+    }
+
+
+def create_test_runbook(**kw):
+    """Create a runbook in the DB and return Runbook model.
+
+    :param kw: kwargs with overriding values for the runbook.
+    :returns: Test Runbook DB object.
+    """
+    runbook = get_test_runbook(**kw)
+    dbapi = db_api.get_instance()
+    # Let DB generate an ID if one isn't specified explicitly.
+    if 'id' not in kw:
+        del runbook['id']
+    if 'steps' not in kw:
+        for step in runbook['steps']:
+            del step['id']
+            del step['runbook_id']
+    else:
+        for kw_step, runbook_step in zip(kw['steps'], runbook['steps']):
+            if 'id' not in kw_step:
+                del runbook_step['id']
+    return dbapi.create_runbook(runbook)
+
+
 def get_test_history(**kw):
     return {
         'id': kw.get('id', 345),
@@ -721,3 +755,69 @@ def create_test_history(**kw):
         del history['id']
     dbapi = db_api.get_instance()
     return dbapi.create_node_history(history)
+
+
+def get_test_inventory(**kw):
+    return {
+        'id': kw.get('id', 345),
+        'version': kw.get('version', node_inventory.NodeInventory.VERSION),
+        'node_id': kw.get('node_id', 123),
+        'inventory_data': kw.get('inventory', {"inventory": "test"}),
+        'plugin_data': kw.get('plugin_data', {"pdata": {"plugin": "data"}}),
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at'),
+    }
+
+
+def create_test_inventory(**kw):
+    """Create test inventory entry in DB and return NodeInventory DB object.
+
+    :param kw: kwargs with overriding values for inventory attributes.
+    :returns: Test NodeInventory DB object.
+    """
+    inventory = get_test_inventory(**kw)
+    # Let DB generate ID if it isn't specified explicitly
+    if 'id' not in kw:
+        del inventory['id']
+    dbapi = db_api.get_instance()
+    return dbapi.create_node_inventory(inventory)
+
+
+def create_test_firmware_component(**kw):
+    """Create test Firmware Component entry in DB and return object.
+
+    Function to be used to create test FirmwareComponent object in the DB.
+
+    :param kw: kwargs with overriding values for firmware component
+    :returns: Test FirmwareComponent DB object
+    """
+    fw_cmp_values = get_test_firmware_component(**kw)
+    if 'id' not in kw:
+        del fw_cmp_values['id']
+    dbapi = db_api.get_instance()
+    return dbapi.create_firmware_component(fw_cmp_values)
+
+
+def get_test_firmware_component(**kw):
+    # NOTE(iurygregory): update version to get from the object in the object
+    # patch.
+    return {
+        'id': kw.get('id', 256),
+        'node_id': kw.get('node_id', 123),
+        'component': kw.get('component', 'bmc'),
+        'initial_version': kw.get('initial_version', 'v1.0.0'),
+        'current_version': kw.get('current_version', 'v1.0.0'),
+        'last_version_flashed': kw.get('last_version_flashed', None),
+        'version': kw.get('version', firmware.FirmwareComponent.VERSION),
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at'),
+    }
+
+
+def get_test_firmware_component_list():
+    return [
+        {'component': 'bmc', 'initial_version': 'v1.0.0',
+         'current_version': 'v1.0.0', 'last_version_flashed': None},
+        {'component': 'BIOS', 'initial_version': 'v1.5.0',
+         'current_version': 'v1.5.0', 'last_version_flashed': None},
+    ]
